@@ -40,15 +40,10 @@ these objects be simple dictionaries.
 
 """
 
-import datetime as dt
-
-from oslo_config import cfg
 from oslo_db import api as db_api
 from oslo_db import options as db_options
-import six
-
-from rally.common.i18n import _
 
+from rally.common import cfg
 
 CONF = cfg.CONF
 
@@ -59,36 +54,6 @@ db_options.set_defaults(CONF, connection="sqlite:////tmp/rally.sqlite")
 IMPL = None
 
 
-def serialize(fn):
-    def conv(data):
-        if data is None:
-            return None
-        if isinstance(data, (six.integer_types,
-                             six.string_types,
-                             six.text_type,
-                             dt.date,
-                             dt.time,
-                             float,
-                             )):
-            return data
-        if isinstance(data, dict):
-            return {k: conv(v) for k, v in data.items()}
-        if isinstance(data, (list, tuple)):
-            return [conv(i) for i in data]
-        if hasattr(data, "_as_dict"):
-            result = data._as_dict()
-            for k, v in result.items():
-                result[k] = conv(v)
-            return result
-
-        raise ValueError(_("Can not serialize %s") % data)
-
-    def wrapper(*args, **kwargs):
-        result = fn(*args, **kwargs)
-        return conv(result)
-    return wrapper
-
-
 def get_impl():
     global IMPL
 
@@ -129,40 +94,27 @@ def schema_stamp(revision):
     return get_impl().schema_stamp(revision)
 
 
-def task_get(uuid):
+def task_get(uuid, detailed=False):
     """Returns task by uuid.
 
     :param uuid: UUID of the task.
-    :raises TaskNotFound: if the task does not exist.
+    :param detailed: whether return results of task or not (Defaults to False).
+    :raises DBRecordNotFound: if the task does not exist.
     :returns: task dict with data on the task.
     """
-    return get_impl().task_get(uuid)
+    return get_impl().task_get(uuid, detailed=detailed)
 
 
 def task_get_status(uuid):
     """Returns task by uuid.
 
     :param uuid: UUID of the task.
-    :raises TaskNotFound: if the task does not exist.
+    :raises DBRecordNotFound: if the task does not exist.
     :returns: task dict with data on the task.
     """
     return get_impl().task_get_status(uuid)
 
 
-def task_get_detailed_last():
-    """Returns the most recently created task."""
-    return get_impl().task_get_detailed_last()
-
-
-def task_get_detailed(uuid):
-    """Returns task with results by uuid.
-
-    :param uuid: UUID of the task.
-    :returns: task dict with data on the task and its results.
-    """
-    return get_impl().task_get_detailed(uuid)
-
-
 def task_create(values):
     """Create task record in DB.
 
@@ -177,7 +129,7 @@ def task_update(uuid, values):
 
     :param uuid: UUID of the task.
     :param values: dict with record values.
-    :raises TaskNotFound: if the task does not exist.
+    :raises DBRecordNotFound: if task is not found.
     :returns: new updated task dict with data on the task.
     """
     return get_impl().task_update(uuid, values)
@@ -189,7 +141,8 @@ def task_update_status(task_uuid, status, allowed_statuses):
     :param task_uuid: string with UUID of Task instance.
     :param status: new value to be written into db instead of status.
     :param allowed_statuses: list of expected statuses to update in db.
-    :raises RallyException: if task not found with specified status.
+    :raises DBConflict: if task is in improper status.
+    :raises DBRecordNotFound: if task is not found.
     :returns: the count of rows match as returned by the database's
               "row count" feature
     """
@@ -197,17 +150,19 @@ def task_update_status(task_uuid, status, allowed_statuses):
                                          status)
 
 
-def task_list(status=None, deployment=None):
+def task_list(status=None, env=None, tags=None):
     """Get a list of tasks.
 
     :param status: Task status to filter the returned list on. If set to
-                   None, all the tasks will be returned.
-    :param deployment: Deployment UUID to filter the returned list on.
-                      If set to None, tasks from all deployments will be
-                      returned.
+        None, all the tasks will be returned.
+    :param env: Environment UUID to filter the returned list on.
+        If set to None, tasks from all environments will be returned.
+    :param tags: A list of tags to filter tasks by.
     :returns: A list of dicts with data on the tasks.
     """
-    return get_impl().task_list(status=status, deployment=deployment)
+    return get_impl().task_list(status=status,
+                                env=env,
+                                tags=tags)
 
 
 def task_delete(uuid, status=None):
@@ -218,43 +173,65 @@ def task_delete(uuid, status=None):
     statuses are equal otherwise an exception is raised.
 
     :param uuid: UUID of the task.
-    :raises TaskNotFound: if the task does not exist.
-    :raises TaskInvalidStatus: if the status of the task does not
-                               equal to the status argument.
+    :raises DBRecordNotFound: if the task does not exist.
+    :raises DBConflict: if the status of the task does not
+                        equal to the status argument.
     """
     return get_impl().task_delete(uuid, status=status)
 
 
-def task_result_get_all_by_uuid(task_uuid):
-    """Get list of task results.
-
-    :param task_uuid: string with UUID of Task instance.
-    :returns: list instances of TaskResult.
-    """
-    return get_impl().task_result_get_all_by_uuid(task_uuid)
-
-
-def subtask_create(task_uuid, title, description=None, context=None):
+def subtask_create(task_uuid, title, description=None, contexts=None):
     """Create a subtask.
 
     :param task_uuid: string with UUID of Task instance.
-    :param title: subtask title.
-    :param description: subtask description.
-    :param context: subtask context dict.
+    :param title: Subtask title.
+    :param description: string with the description of Subtask
+    :param contexts: a dict with config of Workload contexts
     :returns: a dict with data on the subtask.
     """
-    return get_impl().subtask_create(task_uuid, title, description, context)
+    return get_impl().subtask_create(task_uuid, title, description, contexts)
 
 
-def workload_create(task_uuid, subtask_uuid, key):
+def subtask_update(subtask_uuid, values):
+    """Update a subtask.
+
+    :param subtask_uuid: string with UUID of Subtask instance.
+    :param values: dict with record values.
+    :returns: a dict with data on the subtask.
+    """
+    return get_impl().subtask_update(subtask_uuid, values)
+
+
+def workload_create(task_uuid, subtask_uuid, name, description, position,
+                    runner, runner_type, hooks, contexts, sla, args):
     """Create a workload.
 
     :param task_uuid: string with UUID of Task instance.
     :param subtask_uuid: string with UUID of Subtask instance.
-    :param key: dict with record values on the workload.
+    :param name: string with the name of Workload
+    :param description: string with the description of Workload
+    :param position: integer with an order of Workload in Subtask
+    :param runner: a dict with config of Workload runner
+    :param runner_type: a type of Workload runner
+    :param hooks: a list with Workload hooks config and results
+    :param contexts: a dict with config of Workload contexts
+    :param sla: a dict with config of Workload sla
+    :param args: a dict with arguments of Workload
     :returns: a dict with data on the workload.
     """
-    return get_impl().workload_create(task_uuid, subtask_uuid, key)
+    return get_impl().workload_create(
+        task_uuid, subtask_uuid, name=name, description=description,
+        position=position, runner=runner, runner_type=runner_type,
+        hooks=hooks, contexts=contexts, sla=sla, args=args)
+
+
+def workload_get(workload_uuid):
+    """Get a workload.
+
+    :param workload_uuid: string with UUID of Workload to fetch.
+    :returns: a dict with data on the workload.
+    """
+    return get_impl().workload_get(workload_uuid)
 
 
 def workload_data_create(task_uuid, workload_uuid, chunk_order, data):
@@ -270,108 +247,104 @@ def workload_data_create(task_uuid, workload_uuid, chunk_order, data):
                                            chunk_order, data)
 
 
-def workload_set_results(workload_uuid, data):
+def workload_set_results(workload_uuid, subtask_uuid, task_uuid, load_duration,
+                         full_duration, start_time, sla_results,
+                         contexts_results, hooks_results=None):
     """Set workload results.
 
     :param workload_uuid: string with UUID of Workload instance.
-    :param data: dict with workload results.
+    :param subtask_uuid: string with UUID of Workload's parent Subtask.
+    :param task_uuid: string with UUID of Workload's parent Task.
+    :param load_duration: float value of Workload's load duration
+    :param full_duration: float value of Workload's full duration (
+        generating load, executing contexts and etc)
+    :param start_time: a timestamp of load start
+    :param sla_results: a list with Workload's SLA results
+    :param contexts_results: a list with Contexts execution results
+    :param hooks_results: a list with Workload's Hooks results
     :returns: a dict with data on the workload.
     """
-    return get_impl().workload_set_results(workload_uuid, data)
+    return get_impl().workload_set_results(
+        workload_uuid=workload_uuid,
+        subtask_uuid=subtask_uuid,
+        task_uuid=task_uuid,
+        load_duration=load_duration,
+        full_duration=full_duration,
+        start_time=start_time,
+        sla_results=sla_results,
+        hooks_results=hooks_results,
+        contexts_results=contexts_results)
 
 
-def deployment_create(values):
-    """Create a deployment from the values dictionary.
-
-    :param values: dict with record values on the deployment.
-    :returns: a dict with data on the deployment.
-    """
-    return get_impl().deployment_create(values)
+def env_get(uuid_or_name):
+    """Returns envs with corresponding uuid or name."""
+    return get_impl().env_get(uuid_or_name)
 
 
-def deployment_delete(uuid):
-    """Delete a deployment by UUID.
-
-    :param uuid: UUID of the deployment.
-    :raises DeploymentNotFound: if the deployment does not exist.
-    :raises DeploymentIsBusy: if the resource is not enough.
-    """
-    return get_impl().deployment_delete(uuid)
+def env_get_status(uuid):
+    """Returns status of env with corresponding uuid."""
+    return get_impl().env_get_status(uuid)
 
 
-def deployment_get(deployment):
-    """Get a deployment by UUID.
-
-    :param deployment: UUID or name of the deployment.
-    :raises DeploymentNotFound: if the deployment does not exist.
-    :returns: a dict with data on the deployment.
-    """
-    return get_impl().deployment_get(deployment)
+def env_list(status=None):
+    """Return list of envs, filtered by status, if status provided."""
+    return get_impl().env_list(status=status)
 
 
-def deployment_update(uuid, values):
-    """Update a deployment by values.
-
-    :param uuid: UUID of the deployment.
-    :param values: dict with items to update.
-    :raises DeploymentNotFound: if the deployment does not exist.
-    :returns: a dict with data on the deployment.
-    """
-    return get_impl().deployment_update(uuid, values)
+def env_create(name, status, description, extras, config, spec, platforms):
+    """Created db record of env and platforms."""
+    return get_impl().env_create(
+        name, status, description, extras, config, spec, platforms)
 
 
-def deployment_list(status=None, parent_uuid=None, name=None):
-    """Get list of deployments.
-
-    :param status: if None returns any deployments with any status.
-    :param parent_uuid: filter by parent. If None, return only "root"
-                        deployments.
-    :param name: name of deployment.
-    :returns: a list of dicts with data on the deployments.
-    """
-    return get_impl().deployment_list(status=status, parent_uuid=parent_uuid,
-                                      name=name)
+def env_rename(uuid, old_name, new_name):
+    """Renames env. Returns op result as bool"""
+    return get_impl().env_rename(uuid, old_name, new_name)
 
 
-def resource_create(values):
-    """Create a resource from the values dictionary.
-
-    :param values: a dict with data on the resource.
-    :returns: a dict with updated data on the resource.
-    """
-    return get_impl().resource_create(values)
+def env_update(uuid, description=None, extras=None, config=None):
+    """Update description and extra of envs. Returns op result as bool."""
+    return get_impl().env_update(
+        uuid, description=description, extras=extras, config=config)
 
 
-def resource_get_all(deployment_uuid, provider_name=None, type=None):
-    """Return resources of a deployment.
-
-    :param deployment_uuid: filter by uuid of a deployment
-    :param provider_name: filter by provider_name, if is None, then
-                          return all providers
-    :param type: filter by type, if is None, then return all types
-    :returns: a list of dicts with data on a resource
-    """
-    return get_impl().resource_get_all(deployment_uuid,
-                                       provider_name=provider_name,
-                                       type=type)
+def env_set_status(uuid, old_status, new_status):
+    """Set new env status. """
+    return get_impl().env_set_status(uuid, old_status, new_status)
 
 
-def resource_delete(id):
-    """Delete a resource.
-
-    :param id: ID of a resource.
-    :raises ResourceNotFound: if the resource does not exist.
-    """
-    return get_impl().resource_delete(id)
+def env_delete_cascade(uuid):
+    """Delete envs, platforms and all related to env resources."""
+    return get_impl().env_delete_cascade(uuid)
 
 
-def verifier_create(name, vtype, namespace, source, version, system_wide,
+def platforms_list(env_uuid):
+    """List platforms related to some env."""
+    return get_impl().platforms_list(env_uuid)
+
+
+def platform_get(uuid):
+    """Returns platforms with corresponding uuid."""
+    return get_impl().platform_get(uuid)
+
+
+def platform_set_status(uuid, old_status, new_status):
+    """Set's new status to platform"""
+    return get_impl().platform_set_status(uuid, old_status, new_status)
+
+
+def platform_set_data(uuid, platform_data=None, plugin_data=None):
+    """Set's platform data."""
+    return get_impl().platform_set_data(uuid, platform_data, plugin_data)
+
+
+def verifier_create(name, vtype, platform, source, version, system_wide,
                     extra_settings=None):
     """Create a verifier record.
 
     :param name: verifier name
     :param vtype: verifier plugin name
-    :param namespace: verifier plugin namespace
+    :param platform: verifier plugin platform
     :param source: path or URL to a verifier repo
     :param version: branch, tag or commit ID of a verifier repo
     :param system_wide: whether or not to use the system-wide environment
@@ -379,7 +352,7 @@ def verifier_create(name, vtype, namespace, source, version, system_wide,
     :returns: a dict with verifier data
     """
     return get_impl().verifier_create(name=name, vtype=vtype,
-                                      namespace=namespace, source=source,
+                                      platform=platform, source=source,
                                       version=version, system_wide=system_wide,
                                       extra_settings=extra_settings)
 
@@ -388,7 +361,7 @@ def verifier_get(verifier_id):
     """Get a verifier record.
 
     :param verifier_id: verifier name or UUID
-    :raises ResourceNotFound: if verifier does not exist
+    :raises DBRecordNotFound: if verifier does not exist
     :returns: a dict with verifier data
     """
     return get_impl().verifier_get(verifier_id)
@@ -407,7 +380,7 @@ def verifier_delete(verifier_id):
     """Delete a verifier record.
 
     :param verifier_id: verifier name or UUID
-    :raises ResourceNotFound: if verifier does not exist
+    :raises DBRecordNotFound: if verifier does not exist
     """
     get_impl().verifier_delete(verifier_id)
 
@@ -417,55 +390,58 @@ def verifier_update(verifier_id, **properties):
 
     :param verifier_id: verifier name or UUID
     :param properties: a dict with new properties to update verifier record
-    :raises ResourceNotFound: if verifier does not exist
+    :raises DBRecordNotFound: if verifier does not exist
     :returns: the updated dict with verifier data
     """
     return get_impl().verifier_update(verifier_id, properties)
 
 
-def verification_create(verifier_uuid, deployment_uuid, tags=None,
-                        run_args=None):
+def verification_create(verifier_uuid, env, tags=None, run_args=None):
     """Create a verification record.
 
     :param verifier_uuid: verification UUID
-    :param deployment_uuid: deployment UUID
+    :param env: Environment UUID
     :param tags: a list of tags to assign them to verification
     :param run_args: a dict with run arguments for verification
     :returns: a dict with verification data
     """
-    return get_impl().verification_create(verifier_uuid, deployment_uuid,
-                                          tags, run_args)
+    return get_impl().verification_create(verifier_uuid,
+                                          env=env,
+                                          tags=tags,
+                                          run_args=run_args)
 
 
 def verification_get(verification_uuid):
     """Get a verification record.
 
     :param verification_uuid: verification UUID
-    :raises ResourceNotFound: if verification does not exist
+    :raises DBRecordNotFound: if verification does not exist
     :returns: a dict with verification data
     """
     return get_impl().verification_get(verification_uuid)
 
 
-def verification_list(verifier_id=None, deployment_id=None, tags=None,
+def verification_list(verifier_id=None, env=None, tags=None,
                       status=None):
     """List all verification records.
 
     :param verifier_id: verifier name or UUID to filter verifications by
-    :param deployment_id: deployment name or UUID to filter verifications by
+    :param env: Environment name or UUID to filter verifications by
     :param tags: tags to filter verifications by
     :param status: status to filter verifications by
     :returns: a list of dicts with verifications data
     """
-    return get_impl().verification_list(verifier_id, deployment_id, tags,
-                                        status)
+    return get_impl().verification_list(verifier_id,
+                                        env=env,
+                                        tags=tags,
+                                        status=status)
 
 
 def verification_delete(verification_uuid):
     """Delete a verification record.
 
     :param verification_uuid: verification UUID
-    :raises ResourceNotFound: if verification does not exist
+    :raises DBRecordNotFound: if verification does not exist
     """
     return get_impl().verification_delete(verification_uuid)
 
@@ -475,49 +451,7 @@ def verification_update(uuid, **properties):
 
     :param uuid: verification UUID
     :param properties: a dict with new properties to update verification record
-    :raises ResourceNotFound: if verification does not exist
+    :raises DBRecordNotFound: if verification does not exist
     :returns: the updated dict with verification data
     """
     return get_impl().verification_update(uuid, properties)
-
-
-def register_worker(values):
-    """Register a new worker service at the specified hostname.
-
-    :param values: A dict of values which must contain the following:
-                   {
-                    "hostname": the unique hostname which identifies
-                                this worker service.
-                   }
-    :returns: A worker.
-    :raises WorkerAlreadyRegistered: if worker already registered
-    """
-    return get_impl().register_worker(values)
-
-
-def get_worker(hostname):
-    """Retrieve a worker service record from the database.
-
-    :param hostname: The hostname of the worker service.
-    :returns: A worker.
-    :raises WorkerNotFound: if worker not found
-    """
-    return get_impl().get_worker(hostname)
-
-
-def unregister_worker(hostname):
-    """Unregister this worker with the service registry.
-
-    :param hostname: The hostname of the worker service.
-    :raises WorkerNotFound: if worker not found
-    """
-    get_impl().unregister_worker(hostname)
-
-
-def update_worker(hostname):
-    """Mark a worker as active by updating its "updated_at" property.
-
-    :param hostname: The hostname of this worker service.
-    :raises WorkerNotFound: if worker not found
-    """
-    get_impl().update_worker(hostname)
