@@ -25,15 +25,13 @@ import alembic
 from alembic import config as alembic_config
 import alembic.migration as alembic_migration
 from alembic import script as alembic_script
-from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_db.sqlalchemy import session as db_session
-from oslo_utils import timeutils
 import six
-from sqlalchemy import or_
-from sqlalchemy.orm.exc import NoResultFound
-from sqlalchemy.orm import load_only as sa_loadonly
+import sqlalchemy as sa
+import sqlalchemy.orm   # noqa
 
+from rally.common import cfg
 from rally.common.db.sqlalchemy import models
 from rally import consts
 from rally import exceptions
@@ -76,7 +74,7 @@ def serialize_data(data):
                 result[key] = serialize_data(getattr(data, key))
         return result
 
-    raise ValueError("Can not serialize %s" % data)
+    raise exceptions.DBException("Can not serialize %s" % data)
 
 
 def serialize(fn):
@@ -208,16 +206,15 @@ class Connection(object):
         :raises Exception: when the model is not a sublcass of
                  :class:`rally.common.db.sqlalchemy.models.RallyBase`.
         """
-        session = session or get_session()
-        query = session.query(model)
-
         def issubclassof_rally_base(obj):
             return isinstance(obj, type) and issubclass(obj, models.RallyBase)
 
         if not issubclassof_rally_base(model):
-            raise Exception("The model should be a subclass of RallyBase")
+            raise exceptions.DBException(
+                "The model %s should be a subclass of RallyBase" % model)
 
-        return query
+        session = session or get_session()
+        return session.query(model)
 
     def _tags_get(self, uuid, tag_type, session=None):
         tags = (self.model_query(models.Tag, session=session).
@@ -235,11 +232,12 @@ class Connection(object):
     def _task_get(self, uuid, load_only=None, session=None):
         pre_query = self.model_query(models.Task, session=session)
         if load_only:
-            pre_query = pre_query.options(sa_loadonly(load_only))
+            pre_query = pre_query.options(sa.orm.load_only(load_only))
 
         task = pre_query.filter_by(uuid=uuid).first()
         if not task:
-            raise exceptions.TaskNotFound(uuid=uuid)
+            raise exceptions.DBRecordNotFound(
+                criteria="uuid: %s" % uuid, table="tasks")
         task.tags = sorted(self._tags_get(uuid, consts.TagType.TASK, session))
         return task
 
@@ -323,15 +321,14 @@ class Connection(object):
             update({"status": status_value}, synchronize_session=False)
         )
         if not result:
-            status = " or ".join(statuses)
-            msg = ("Task with uuid='%(uuid)s' and in statuses:'"
-                   "%(statuses)s' not found.'") % {"uuid": uuid,
-                                                   "statuses": status}
-            raise exceptions.RallyException(msg)
+            raise exceptions.DBRecordNotFound(
+                criteria="uuid=%(uuid)s and status in [%(statuses)s]"
+                         % {"uuid": uuid, "statuses": ", ".join(statuses)},
+                table="tasks")
         return result
 
     @serialize
-    def task_list(self, status=None, deployment=None, tags=None):
+    def task_list(self, status=None, env=None, tags=None):
         session = get_session()
         tasks = []
         with session.begin():
@@ -340,9 +337,8 @@ class Connection(object):
             filters = {}
             if status is not None:
                 filters["status"] = status
-            if deployment is not None:
-                filters["deployment_uuid"] = self.deployment_get(
-                    deployment)["uuid"]
+            if env is not None:
+                filters["env_uuid"] = self.env_get(env)["uuid"]
             if filters:
                 query = query.filter_by(**filters)
 
@@ -386,10 +382,14 @@ class Connection(object):
                 if status is not None:
                     task = base_query.first()
                     if task:
-                        raise exceptions.TaskInvalidStatus(uuid=uuid,
-                                                           require=status,
-                                                           actual=task.status)
-                raise exceptions.TaskNotFound(uuid=uuid)
+                        raise exceptions.DBConflict(
+                            "Task `%(uuid)s` in `%(actual)s` status but "
+                            "`%(require)s` is required."
+                            % {"uuid": uuid,
+                               "require": status, "actual": task.status})
+
+                raise exceptions.DBRecordNotFound(
+                    criteria="uuid: %s" % uuid, table="tasks")
 
     def _subtasks_get_all_by_task_uuid(self, task_uuid, session=None):
         result = (self.model_query(models.Subtask, session=session).filter_by(
@@ -408,12 +408,13 @@ class Connection(object):
         return subtasks
 
     @serialize
-    def subtask_create(self, task_uuid, title, description=None, context=None):
+    def subtask_create(self, task_uuid, title, description=None,
+                       contexts=None):
         subtask = models.Subtask(task_uuid=task_uuid)
         subtask.update({
             "title": title,
             "description": description or "",
-            "context": context or {},
+            "contexts": contexts or {},
         })
         subtask.save()
         return subtask
@@ -433,8 +434,8 @@ class Connection(object):
 
     @serialize
     def workload_create(self, task_uuid, subtask_uuid, name, description,
-                        position, runner, runner_type, hooks, context, sla,
-                        args, context_execution, statistics):
+                        position, runner, runner_type, hooks, contexts, sla,
+                        args):
         workload = models.Workload(task_uuid=task_uuid,
                                    subtask_uuid=subtask_uuid,
                                    name=name,
@@ -443,11 +444,9 @@ class Connection(object):
                                    runner=runner,
                                    runner_type=runner_type,
                                    hooks=hooks,
-                                   context=context,
+                                   contexts=contexts or {},
                                    sla=sla,
-                                   args=args,
-                                   context_execution=context_execution,
-                                   statistics=statistics)
+                                   args=args)
         workload.save()
         return workload
 
@@ -503,7 +502,7 @@ class Connection(object):
     @serialize
     def workload_set_results(self, workload_uuid, subtask_uuid, task_uuid,
                              load_duration, full_duration, start_time,
-                             sla_results, hooks_results):
+                             sla_results, hooks_results, contexts_results):
         session = get_session()
         with session.begin():
             workload_results = self._task_workload_data_get_all(workload_uuid)
@@ -542,7 +541,7 @@ class Connection(object):
                 uuid=workload_uuid).update(
                 {
                     "sla_results": {"sla": sla},
-                    "context_execution": {},
+                    "contexts_results": contexts_results,
                     "hooks": hooks_results or [],
                     "load_duration": load_duration,
                     "full_duration": full_duration,
@@ -569,88 +568,148 @@ class Connection(object):
             session.query(models.Subtask).filter_by(uuid=subtask_uuid).update(
                 subtask_values)
 
-    def _deployment_get(self, deployment, session=None):
-        stored_deployment = self.model_query(
-            models.Deployment,
-            session=session).filter_by(name=deployment).first()
-        if not stored_deployment:
-            stored_deployment = self.model_query(
-                models.Deployment,
-                session=session).filter_by(uuid=deployment).first()
-
-        if not stored_deployment:
-            raise exceptions.DeploymentNotFound(deployment=deployment)
-        return stored_deployment
+    @serialize
+    def env_get(self, uuid_or_name):
+        env = (self.model_query(models.Env)
+                   .filter(sa.or_(models.Env.uuid == uuid_or_name,
+                                  models.Env.name == uuid_or_name))
+                   .first())
+        if not env:
+            raise exceptions.DBRecordNotFound(
+                criteria="uuid or name is %s" % uuid_or_name, table="envs")
+        return env
 
     @serialize
-    def deployment_create(self, values):
-        deployment = models.Deployment()
-        try:
-            deployment.update(values)
-            deployment.save()
-        except db_exc.DBDuplicateEntry:
-            raise exceptions.DeploymentNameExists(deployment=values["name"])
-        return deployment
-
-    def deployment_delete(self, uuid):
-        session = get_session()
-        with session.begin():
-            count = (self.model_query(models.Resource, session=session).
-                     filter_by(deployment_uuid=uuid).count())
-            if count:
-                raise exceptions.DeploymentIsBusy(uuid=uuid)
-
-            count = (self.model_query(models.Deployment, session=session).
-                     filter_by(uuid=uuid).delete(synchronize_session=False))
-            if not count:
-                raise exceptions.DeploymentNotFound(deployment=uuid)
+    def env_get_status(self, uuid):
+        resp = (self.model_query(models.Env)
+                    .filter_by(uuid=uuid)
+                    .options(sa.orm.load_only("status"))
+                    .first())
+        if not resp:
+            raise exceptions.DBRecordNotFound(
+                criteria="uuid: %s" % uuid, table="envs")
+        return resp["status"]
 
     @serialize
-    def deployment_get(self, deployment):
-        return self._deployment_get(deployment)
-
-    @serialize
-    def deployment_update(self, deployment, values):
-        session = get_session()
-        values.pop("uuid", None)
-        with session.begin():
-            dpl = self._deployment_get(deployment, session=session)
-            dpl.update(values)
-        return dpl
-
-    @serialize
-    def deployment_list(self, status=None, parent_uuid=None, name=None):
-        query = (self.model_query(models.Deployment).
-                 filter_by(parent_uuid=parent_uuid))
-
-        if name:
-            query = query.filter_by(name=name)
+    def env_list(self, status=None):
+        query = self.model_query(models.Env)
         if status:
             query = query.filter_by(status=status)
         return query.all()
 
     @serialize
-    def resource_create(self, values):
-        resource = models.Resource()
-        resource.update(values)
-        resource.save()
-        return resource
+    def env_create(self, name, status, description, extras, config,
+                   spec, platforms):
+        try:
+            env_uuid = models.UUID()
+            for p in platforms:
+                p["env_uuid"] = env_uuid
+
+            env = models.Env(
+                name=name, uuid=env_uuid,
+                status=status, description=description,
+                extras=extras, config=config, spec=spec
+            )
+            get_session().bulk_save_objects([env] + [
+                models.Platform(**p) for p in platforms
+            ])
+        except db_exc.DBDuplicateEntry:
+            raise exceptions.DBRecordExists(
+                field="name", value=name, table="envs")
+
+        return self.env_get(env_uuid)
+
+    def env_rename(self, uuid, old_name, new_name):
+        try:
+            return bool(self.model_query(models.Env)
+                            .filter_by(uuid=uuid, name=old_name)
+                            .update({"name": new_name}))
+        except db_exc.DBDuplicateEntry:
+            raise exceptions.DBRecordExists(
+                field="name", value=new_name, table="envs")
+
+    def env_update(self, uuid, description=None, extras=None, config=None):
+        values = {}
+        if description is not None:
+            values["description"] = description
+        if extras is not None:
+            values["extras"] = extras
+        if config is not None:
+            values["config"] = config
+
+        if not values:
+            return True
+
+        return bool(self.model_query(models.Env)
+                        .filter_by(uuid=uuid)
+                        .update(values))
+
+    def env_set_status(self, uuid, old_status, new_status):
+        count = (self.model_query(models.Env)
+                     .filter_by(uuid=uuid, status=old_status)
+                     .update({"status": new_status}))
+        if count:
+            return True
+
+        raise exceptions.DBConflict(
+            "Env %s should be in status %s actual %s"
+            % (uuid, old_status, self.env_get_status(uuid)))
+
+    def env_delete_cascade(self, uuid):
+        session = get_session()
+        with session.begin():
+            (self.model_query(models.Task, session=session)
+                .filter_by(env_uuid=uuid)
+                .delete())
+            (self.model_query(models.Verification, session=session)
+                .filter_by(env_uuid=uuid)
+                .delete())
+            (self.model_query(models.Platform, session=session)
+                 .filter_by(env_uuid=uuid)
+                 .delete())
+            (self.model_query(models.Env, session=session)
+                 .filter_by(uuid=uuid)
+                 .delete())
 
     @serialize
-    def resource_get_all(self, deployment_uuid, provider_name=None, type=None):
-        query = (self.model_query(models.Resource).
-                 filter_by(deployment_uuid=deployment_uuid))
-        if provider_name is not None:
-            query = query.filter_by(provider_name=provider_name)
-        if type is not None:
-            query = query.filter_by(type=type)
-        return query.all()
+    def platforms_list(self, env_uuid):
+        return (self.model_query(models.Platform)
+                    .filter_by(env_uuid=env_uuid)
+                    .all())
 
-    def resource_delete(self, id):
-        count = (self.model_query(models.Resource).
-                 filter_by(id=id).delete(synchronize_session=False))
-        if not count:
-            raise exceptions.ResourceNotFound(id=id)
+    @serialize
+    def platform_get(self, uuid):
+        p = self.model_query(models.Platform).filter_by(uuid=uuid).first()
+        if not p:
+            raise exceptions.DBRecordNotFound(
+                criteria="uuid = %s" % uuid, table="platforms")
+        return p
+
+    def platform_set_status(self, uuid, old_status, new_status):
+        count = (self.model_query(models.Platform)
+                     .filter_by(uuid=uuid, status=old_status)
+                     .update({"status": new_status}))
+        if count:
+            return True
+
+        platform = self.platform_get(uuid)
+        raise exceptions.DBConflict(
+            "Platform %s should be in status %s actual %s"
+            % (uuid, old_status, platform["status"]))
+
+    def platform_set_data(self, uuid, platform_data=None, plugin_data=None):
+        values = {}
+        if platform_data is not None:
+            values["platform_data"] = platform_data
+        if plugin_data is not None:
+            values["plugin_data"] = plugin_data
+
+        if not values:
+            return True
+
+        return bool(self.model_query(models.Platform)
+                        .filter_by(uuid=uuid)
+                        .update(values))
 
     @serialize
     def verifier_create(self, name, vtype, platform, source, version,
@@ -670,10 +729,11 @@ class Connection(object):
     def _verifier_get(self, verifier_id, session=None):
         verifier = self.model_query(
             models.Verifier, session=session).filter(
-                or_(models.Verifier.name == verifier_id,
-                    models.Verifier.uuid == verifier_id)).first()
+                sa.or_(models.Verifier.name == verifier_id,
+                       models.Verifier.uuid == verifier_id)).first()
         if not verifier:
-            raise exceptions.ResourceNotFound(id=verifier_id)
+            raise exceptions.DBRecordNotFound(
+                criteria="name or uuid is %s" % verifier_id, table="verifiers")
         return verifier
 
     @serialize
@@ -688,11 +748,13 @@ class Connection(object):
         with session.begin():
             query = self.model_query(
                 models.Verifier, session=session).filter(
-                    or_(models.Verifier.name == verifier_id,
-                        models.Verifier.uuid == verifier_id))
+                    sa.or_(models.Verifier.name == verifier_id,
+                           models.Verifier.uuid == verifier_id))
             count = query.delete(synchronize_session=False)
             if not count:
-                raise exceptions.ResourceNotFound(id=verifier_id)
+                raise exceptions.DBRecordNotFound(
+                    criteria="name or uuid is %s" % verifier_id,
+                    table="verifiers")
 
     @serialize
     def verifier_update(self, verifier_id, properties):
@@ -704,13 +766,13 @@ class Connection(object):
         return verifier
 
     @serialize
-    def verification_create(self, verifier_id, deployment_id, tags=None,
+    def verification_create(self, verifier_id, env, tags=None,
                             run_args=None):
         verifier = self._verifier_get(verifier_id)
-        deployment = self._deployment_get(deployment_id)
+        env = self.env_get(env)
         verification = models.Verification()
         verification.update({"verifier_uuid": verifier.uuid,
-                             "deployment_uuid": deployment["uuid"],
+                             "env_uuid": env["uuid"],
                              "run_args": run_args})
         verification.save()
 
@@ -736,11 +798,12 @@ class Connection(object):
             models.Verification, session=session).filter_by(
             uuid=verification_uuid).first()
         if not verification:
-            raise exceptions.ResourceNotFound(id=verification_uuid)
+            raise exceptions.DBRecordNotFound(
+                criteria="uuid: %s" % verification_uuid, table="verifications")
         return verification
 
     @serialize
-    def verification_list(self, verifier_id=None, deployment_id=None,
+    def verification_list(self, verifier_id=None, env=None,
                           tags=None, status=None):
         session = get_session()
         with session.begin():
@@ -748,10 +811,9 @@ class Connection(object):
             if verifier_id:
                 verifier = self._verifier_get(verifier_id, session=session)
                 filter_by["verifier_uuid"] = verifier.uuid
-            if deployment_id:
-                deployment = self._deployment_get(deployment_id,
-                                                  session=session)
-                filter_by["deployment_uuid"] = deployment.uuid
+            if env:
+                env = self.env_get(env)
+                filter_by["env_uuid"] = env["uuid"]
             if status:
                 filter_by["status"] = status
 
@@ -779,7 +841,8 @@ class Connection(object):
                 models.Verification, session=session).filter_by(
                 uuid=verification_uuid).delete(synchronize_session=False)
         if not count:
-            raise exceptions.ResourceNotFound(id=verification_uuid)
+            raise exceptions.DBRecordNotFound(
+                criteria="uuid: %s" % verification_uuid, table="verifications")
 
     @serialize
     def verification_update(self, verification_uuid, properties):
@@ -789,36 +852,3 @@ class Connection(object):
             verification.update(properties)
             verification.save()
         return verification
-
-    @serialize
-    def register_worker(self, values):
-        try:
-            worker = models.Worker()
-            worker.update(values)
-            worker.update({"updated_at": timeutils.utcnow()})
-            worker.save()
-            return worker
-        except db_exc.DBDuplicateEntry:
-            raise exceptions.WorkerAlreadyRegistered(
-                worker=values["hostname"])
-
-    @serialize
-    def get_worker(self, hostname):
-        try:
-            return (self.model_query(models.Worker).
-                    filter_by(hostname=hostname).one())
-        except NoResultFound:
-            raise exceptions.WorkerNotFound(worker=hostname)
-
-    def unregister_worker(self, hostname):
-        count = (self.model_query(models.Worker).
-                 filter_by(hostname=hostname).delete())
-        if count == 0:
-            raise exceptions.WorkerNotFound(worker=hostname)
-
-    def update_worker(self, hostname):
-        count = (self.model_query(models.Worker).
-                 filter_by(hostname=hostname).
-                 update({"updated_at": timeutils.utcnow()}))
-        if count == 0:
-            raise exceptions.WorkerNotFound(worker=hostname)
