@@ -20,8 +20,8 @@ import time
 import traceback
 
 import jsonschema
-from oslo_config import cfg
 
+from rally.common import cfg
 from rally.common import logging
 from rally.common import objects
 from rally.common import utils
@@ -52,7 +52,7 @@ class ResultConsumer(object):
     """
 
     def __init__(self, workload_cfg, task, subtask, workload, runner,
-                 abort_on_sla_failure):
+                 abort_on_sla_failure, ctx_manager):
         """ResultConsumer constructor.
 
         :param workload_cfg: A configuration of the Workload
@@ -63,6 +63,7 @@ class ResultConsumer(object):
                        consumed
         :param abort_on_sla_failure: True if the execution should be stopped
                                      when some SLA check fails
+        :param ctx_manager: ContextManager instance
         """
 
         self.task = task
@@ -84,6 +85,7 @@ class ResultConsumer(object):
         self.aborting_checker = threading.Thread(target=self.wait_and_abort)
         if self.workload_cfg["hooks"]:
             self.event_thread = threading.Thread(target=self._consume_events)
+        self._cm = ctx_manager
 
     def __enter__(self):
         self.thread.start()
@@ -176,7 +178,9 @@ class ResultConsumer(object):
         self.workload.set_results(load_duration=load_duration,
                                   full_duration=(self.finish - self.start),
                                   sla_results=self.sla_checker.results(),
-                                  start_time=start_time, **results)
+                                  start_time=start_time,
+                                  contexts_results=self._cm.contexts_results(),
+                                  **results)
 
     @staticmethod
     def is_task_in_aborting_status(task_uuid, check_soft=True):
@@ -227,25 +231,24 @@ class TaskEngine(object):
         Typical usage:
             ...
 
-            engine = TaskEngine(config, task, deployment)
+            engine = TaskEngine(config, task, env)
             engine.validate()   # to test config
             engine.run()        # to run config
     """
 
-    def __init__(self, config, task, deployment,
-                 abort_on_sla_failure=False):
+    def __init__(self, config, task, env, abort_on_sla_failure=False):
         """TaskEngine constructor.
 
         :param config: An instance of a TaskConfig
         :param task: Instance of Task,
                      the current task which is being performed
-        :param deployment: Instance of Deployment,
+        :param env: Instance of Environment,
         :param abort_on_sla_failure: True if the execution should be stopped
                                      when some SLA check fails
         """
         self.config = config
         self.task = task
-        self.deployment = deployment
+        self.env = env
         self.abort_on_sla_failure = abort_on_sla_failure
 
     def _validate_workload(self, workload, vcontext=None, vtype=None):
@@ -336,10 +339,11 @@ class TaskEngine(object):
                               "Task validation of required platforms.")
     def _validate_config_platforms(self, config):
         # FIXME(andreykurilin): prepare the similar context object to others
-        credentials = self.deployment.get_all_credentials()
+        platforms = dict(
+            (p["platform_name"], p["platform_data"])
+            for p in self.env.data["platforms"].values())
         ctx = {"task": self.task,
-               "platforms": dict((p, creds[0])
-                                 for p, creds in credentials.items())}
+               "platforms": platforms}
         for subtask in config.subtasks:
             for workload in subtask["workloads"]:
                 self._validate_workload(
@@ -347,9 +351,30 @@ class TaskEngine(object):
 
     @logging.log_task_wrapper(LOG.info, "Task validation of semantic.")
     def _validate_config_semantic(self, config):
-        self.deployment.verify_connections()
-        validation_ctx = self.deployment.get_validation_context()
-        ctx_obj = {"task": self.task, "config": validation_ctx}
+        LOG.info("Check health of the environment '%s'." % self.env.uuid)
+        failed = []
+        for p, res in self.env.check_health().items():
+            LOG.info("Platform %s (available: %s): %s" %
+                     (p, res["available"], res["message"]))
+            if not res["available"]:
+                failed.append(p)
+                if logging.is_debug():
+                    LOG.error(res["traceback"])
+        if failed:
+            raise exceptions.ValidationError(
+                "One or several platforms are not available: %s. Check logs "
+                "for more details." % ", ".join(failed))
+        validation_ctx = self.env.get_validation_context()
+
+        env_data = self.env.data
+        env_data["platforms"] = dict(
+            (p["platform_name"], p["platform_data"])
+            for p in env_data["platforms"].values())
+
+        ctx_obj = {"task": self.task,
+                   "config": validation_ctx,
+                   "env": env_data}
+
         with context.ContextManager(ctx_obj):
             for subtask in config.subtasks:
                 for workload in subtask["workloads"]:
@@ -388,11 +413,17 @@ class TaskEngine(object):
         for k, v in ctx.items():
             context_config[context.Context.get(k).get_fullname()] = v
 
+        env_data = self.env.data
+        env_data["platforms"] = dict(
+            (p["platform_name"], p["platform_data"])
+            for p in env_data["platforms"].values())
+
         context_obj = {
             "task": self.task,
             "owner_id": owner_id,
             "scenario_name": scenario_name,
-            "config": context_config
+            "config": context_config,
+            "env": env_data
         }
         return context_obj
 
@@ -421,7 +452,7 @@ class TaskEngine(object):
     def _run_subtask(self, subtask):
         subtask_obj = self.task.add_subtask(title=subtask["title"],
                                             description=subtask["description"],
-                                            context=subtask["context"])
+                                            contexts=subtask["contexts"])
 
         try:
             # TODO(astudenov): add subtask context here
@@ -451,7 +482,7 @@ class TaskEngine(object):
             runner=workload["runner"],
             runner_type=workload["runner_type"],
             hooks=workload["hooks"],
-            context=workload["contexts"],
+            contexts=workload["contexts"],
             sla=workload["sla"],
             args=workload["args"])
         workload["uuid"] = workload_obj["uuid"]
@@ -468,9 +499,12 @@ class TaskEngine(object):
         context_obj = self._prepare_context(
             workload["contexts"], workload["name"], workload_obj["uuid"])
         try:
-            with ResultConsumer(workload, self.task, subtask_obj, workload_obj,
-                                runner_obj, self.abort_on_sla_failure):
-                with context.ContextManager(context_obj):
+            ctx_manager = context.ContextManager(context_obj)
+            with ResultConsumer(workload, task=self.task, subtask=subtask_obj,
+                                workload=workload_obj, runner=runner_obj,
+                                abort_on_sla_failure=self.abort_on_sla_failure,
+                                ctx_manager=ctx_manager):
+                with ctx_manager:
                     runner_obj.run(workload["name"], context_obj,
                                    workload["args"])
         except Exception:
@@ -713,7 +747,7 @@ class TaskConfig(object):
 
             # it is not supported feature yet, but the code expects this
             # variable
-            sconf.setdefault("context", {})
+            sconf.setdefault("contexts", {})
 
             workloads = []
             for position, wconf in enumerate(sconf["workloads"]):
