@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import itertools
 import time
 import traceback
@@ -21,7 +22,6 @@ import jsonschema
 from novaclient import exceptions as nova_exc
 import six
 
-from rally.common.i18n import _
 from rally.common import logging
 from rally import consts
 from rally import exceptions
@@ -169,7 +169,8 @@ def wait_is_ready(resource, is_ready, update_resource=None,
                 resource_name=resource_repr,
                 resource_type=resource.__class__.__name__,
                 resource_id=getattr(resource, "id", "<no id>"),
-                resource_status=get_status(resource))
+                resource_status=get_status(resource),
+                timeout=timeout)
 
 
 def wait_for_status(resource, ready_statuses, failure_statuses=None,
@@ -226,9 +227,9 @@ def wait_for_status(resource, ready_statuses, failure_statuses=None,
             delta = current_time - latest_status_update
             LOG.debug(
                 "Waiting for resource %(resource)s. Status changed: "
-                "%(latest)s => %(current)s in %(delta)s" %
-                {"resource": resource_repr, "latest": latest_status,
-                 "current": status, "delta": delta})
+                "%(latest)s => %(current)s in %(delta)s"
+                % {"resource": resource_repr, "latest": latest_status,
+                   "current": status, "delta": delta})
 
             latest_status = status
             latest_status_update = current_time
@@ -248,7 +249,8 @@ def wait_for_status(resource, ready_statuses, failure_statuses=None,
                 resource_name=resource_repr,
                 resource_type=resource.__class__.__name__,
                 resource_id=getattr(resource, id_attr, "<no id>"),
-                resource_status=get_status(resource, status_attr))
+                resource_status=get_status(resource, status_attr),
+                timeout=timeout)
 
 
 @logging.log_deprecated("Use wait_for_status instead.", "0.1.2", once=True)
@@ -279,7 +281,8 @@ def wait_for_delete(resource, update_resource=None, timeout=60,
                 resource_name=getattr(resource, "name", repr(resource)),
                 resource_type=resource.__class__.__name__,
                 resource_id=getattr(resource, "id", "<no id>"),
-                resource_status=get_status(resource))
+                resource_status=get_status(resource),
+                timeout=timeout)
 
 
 def format_exc(exc):
@@ -299,8 +302,8 @@ def check_service_status(client, service_name):
                 if service.status == "enabled" and service.state == "up":
                     return True
     except nova_exc.NotFound:
-        LOG.warning(_("Unable to retrieve a list of available services from "
-                      "nova. Pre-Grizzly OpenStack deployment?"))
+        LOG.warning("Unable to retrieve a list of available services from "
+                    "nova. Pre-Grizzly OpenStack deployment?")
         return False
     return False
 
@@ -395,7 +398,7 @@ class ActionBuilder(object):
         built action. These will be appended to any args set for the
         action via its binding.
         :param kwargs: (optional) Keyword args to pass into each built
-        action. These will be appended to any kwards set for the action
+        action. These will be appended to any kwargs set for the action
         via its binding.
         """
         self.validate(actions)
@@ -410,3 +413,70 @@ class ActionBuilder(object):
                 self._build(binding["action"], times,
                             *(binding["args"] + args), **dft_kwargs))
         return bound_actions
+
+
+# TODO(andreykurilin): We need to implement some wrapper for atomic actions,
+# we can use these wrapper to simulate new and old format.
+class WrapperForAtomicActions(list):
+
+    def __init__(self, atomic_actions, timestamp=0):
+
+        self.timestamp = timestamp
+
+        if isinstance(atomic_actions, list):
+            self.__atomic_actions = atomic_actions
+            self.__old_atomic_actions = self._convert_new_atomic_actions(
+                self.__atomic_actions)
+        else:
+            self.__atomic_actions = self._convert_old_atomic_actions(
+                atomic_actions)
+            self.__old_atomic_actions = atomic_actions
+
+        super(WrapperForAtomicActions, self).__init__(self.__atomic_actions)
+
+    def _convert_old_atomic_actions(self, old_atomic_actions):
+        """Convert atomic actions to new format. """
+        atomic_actions = []
+        started_at = self.timestamp
+        for name, duration in old_atomic_actions.items():
+            finished_at = started_at + duration
+            atomic_actions.append({"name": name,
+                                   "started_at": started_at,
+                                   "finished_at": finished_at,
+                                   "children": []})
+            started_at = finished_at
+        return atomic_actions
+
+    def _convert_new_atomic_actions(self, atomic_actions):
+        """Convert atomic actions to old format. """
+        old_style = collections.OrderedDict()
+        for action in atomic_actions:
+            duration = action["finished_at"] - action["started_at"]
+            if action["name"] in old_style:
+                name_template = action["name"] + " (%i)"
+                i = 2
+                while name_template % i in old_style:
+                    i += 1
+                old_style[name_template % i] = duration
+            else:
+                old_style[action["name"]] = duration
+        return old_style
+
+    def items(self):
+        return self.__old_atomic_actions.items()
+
+    def get(self, name, default=None):
+        return self.__old_atomic_actions.get(name, default)
+
+    def __iter__(self):
+        return iter(self.__atomic_actions)
+
+    def __len__(self):
+        return len(self.__atomic_actions)
+
+    def __getitem__(self, item):
+        if isinstance(item, int):
+            # it is a call to list:
+            return self.__atomic_actions[item]
+        else:
+            return self.__old_atomic_actions[item]
