@@ -26,8 +26,6 @@ import tempfile
 from oslo_utils import encodeutils
 from six.moves import configparser
 
-DEPLOYMENT_FILE = "/tmp/rally_functests_main_deployment.json"
-
 
 class RallyCliError(Exception):
 
@@ -72,12 +70,9 @@ class Rally(object):
         output = rally("deployment list")
 
     """
+    _DEPLOYMENT_CREATE_ARGS = ""
 
-    def __init__(self, fake=False, force_new_db=False):
-        if not os.path.exists(DEPLOYMENT_FILE):
-            subprocess.call(["rally", "--log-file", "/dev/null",
-                             "deployment", "config"],
-                            stdout=open(DEPLOYMENT_FILE, "w"))
+    def __init__(self, force_new_db=False, plugin_path=None):
 
         # NOTE(sskripnick): we should change home dir to avoid races
         # and do not touch any user files in ~/.rally
@@ -116,10 +111,12 @@ class Rally(object):
             self.args = ["rally"]
             subprocess.call(["rally", "db", "recreate"], env=self.env)
 
+        if plugin_path:
+            self.args.extend(["--plugin-paths", os.path.abspath(plugin_path)])
         self.reports_root = os.environ.get("REPORTS_ROOT",
                                            "rally-cli-output-files")
         self._created_files = []
-        self("deployment create --file %s --name MAIN" % DEPLOYMENT_FILE,
+        self("deployment create --name MAIN%s" % self._DEPLOYMENT_CREATE_ARGS,
              write_report=False)
 
     def __del__(self):
@@ -174,7 +171,7 @@ class Rally(object):
 
     def __call__(self, cmd, getjson=False, report_path=None, raw=False,
                  suffix=None, extension=None, keep_old=False,
-                 write_report=True):
+                 write_report=True, no_logs=False):
         """Call rally in the shell
 
         :param cmd: rally command
@@ -189,13 +186,15 @@ class Rally(object):
         if not isinstance(cmd, list):
             cmd = cmd.split(" ")
         try:
-            if getjson:
+            if no_logs or getjson:
                 cmd = self.args + ["--log-file", "/dev/null"] + cmd
+                with open(os.devnull, "w") as DEVNULL:
+                    output = encodeutils.safe_decode(subprocess.check_output(
+                        cmd, stderr=DEVNULL, env=self.env))
             else:
                 cmd = self.args + cmd
-
-            output = encodeutils.safe_decode(subprocess.check_output(
-                cmd, stderr=subprocess.STDOUT, env=self.env))
+                output = encodeutils.safe_decode(subprocess.check_output(
+                    cmd, stderr=subprocess.STDOUT, env=self.env))
 
             if getjson:
                 return json.loads(output)
