@@ -22,11 +22,12 @@ from neutronclient.common import exceptions as neutron_exceptions
 from novaclient import exceptions as nova_exc
 from watcherclient.common.apiclient import exceptions as watcher_exceptions
 
-from rally import consts
 from rally.plugins.openstack.cleanup import resources
 from tests.unit import test
 
 BASE = "rally.plugins.openstack.cleanup.resources"
+GLANCE_V2_PATH = ("rally.plugins.openstack.services.image.glance_v2."
+                  "GlanceV2Service")
 
 
 class SynchronizedDeletionTestCase(test.TestCase):
@@ -105,14 +106,6 @@ class NovaServerTestCase(test.TestCase):
             server.raw_resource.id)
 
 
-class NovaFloatingIPsTestCase(test.TestCase):
-
-    def test_name(self):
-        fips = resources.NovaFloatingIPs()
-        fips.raw_resource = mock.MagicMock()
-        self.assertTrue(fips.name())
-
-
 class NovaFlavorsTestCase(test.TestCase):
 
     @mock.patch("%s.base.ResourceManager._manager" % BASE)
@@ -121,7 +114,7 @@ class NovaFlavorsTestCase(test.TestCase):
         mock_resource_manager__manager().get.side_effect = exc
         flavor = resources.NovaFlavors()
         flavor.raw_resource = mock.MagicMock()
-        self.assertEqual(True, flavor.is_deleted())
+        self.assertTrue(flavor.is_deleted())
 
     @mock.patch("%s.base.ResourceManager._manager" % BASE)
     def test_is_deleted_fail(self, mock_resource_manager__manager):
@@ -145,41 +138,6 @@ class NovaServerGroupsTestCase(test.TestCase):
         self.assertEqual(server_groups, resources.NovaServerGroups().list())
 
 
-class NovaSecurityGroupTestCase(test.TestCase):
-
-    @mock.patch("%s.base.ResourceManager._manager" % BASE)
-    def test_list(self, mock_resource_manager__manager):
-        secgroups = [mock.MagicMock(), mock.MagicMock(), mock.MagicMock()]
-        secgroups[0].name = "a"
-        secgroups[1].name = "b"
-        secgroups[2].name = "default"
-
-        mock_resource_manager__manager().list.return_value = secgroups
-        self.assertSequenceEqual(secgroups[:2],
-                                 resources.NovaSecurityGroup().list())
-
-
-class NovaFloatingIpsBulkTestCase(test.TestCase):
-
-    def test_id(self):
-        ip_range = resources.NovaFloatingIpsBulk()
-        ip_range.raw_resource = mock.MagicMock()
-        self.assertEqual(ip_range.raw_resource.address, ip_range.id())
-
-    def test_name(self):
-        fips = resources.NovaFloatingIpsBulk()
-        fips.raw_resource = mock.MagicMock()
-        self.assertIsNone(fips.name())
-
-
-class NovaNetworksTestCase(test.TestCase):
-
-    def test_name(self):
-        network = resources.NovaNetworks()
-        network.raw_resource = mock.MagicMock()
-        self.assertEqual(network.raw_resource.label, network.name())
-
-
 class EC2MixinTestCase(test.TestCase):
 
     def get_ec2_mixin(self):
@@ -413,26 +371,51 @@ class NeutronV2LoadbalancerTestCase(test.TestCase):
             neutron_lb.id())
 
 
+class NeutronBgpvpnTestCase(test.TestCase):
+
+    def get_neutron_bgpvpn_mixin(self, extensions=None):
+        if extensions is None:
+            extensions = []
+        admin = mock.Mock()
+        neut = resources.NeutronBgpvpn(admin=admin)
+        neut._manager = mock.Mock()
+        neut._manager().list_extensions.return_value = {
+            "extensions": [{"alias": ext} for ext in extensions]
+        }
+        return neut
+
+    def test_list_user(self):
+        neut = self.get_neutron_bgpvpn_mixin(extensions=["bgpvpn"])
+        user_bgpvpns = {"bgpvpns": [{"tenant_id": "foo", "id": "bgpvpn_id"}]}
+        neut._manager().list_bgpvpns.return_value = user_bgpvpns
+
+        bgpvpns_list = neut.list()
+        self.assertEqual("bgpvpn", neut._resource)
+        neut._manager().list_bgpvpns.assert_called_once_with()
+        self.assertEqual(bgpvpns_list, user_bgpvpns["bgpvpns"])
+
+    def test_list_admin(self):
+        neut = self.get_neutron_bgpvpn_mixin(extensions=["bgpvpn"])
+        admin_bgpvpns = {"bgpvpns": [{"tenant_id": "foo", "id": "bgpvpn_id"}]}
+        neut._manager().list_bgpvpns.return_value = admin_bgpvpns
+
+        self.assertEqual("bgpvpn", neut._resource)
+        self.assertEqual(neut.list(), admin_bgpvpns["bgpvpns"])
+
+
 class NeutronFloatingIPTestCase(test.TestCase):
 
     def test_name(self):
-        fips = resources.NeutronFloatingIP({"name": "foo"})
-        self.assertIsInstance(fips.name(), resources.base.NoName)
+        fips = resources.NeutronFloatingIP({"name": "foo",
+                                            "description": "OoO"})
+        self.assertEqual(fips.name(), "OoO")
 
     def test_list(self):
         fips = {"floatingips": [{"tenant_id": "foo", "id": "foo"}]}
 
         user = mock.MagicMock()
-        user.services.return_value = {}
         user.neutron.return_value.list_floatingips.return_value = fips
 
-        self.assertEqual(
-            [], resources.NeutronFloatingIP(user=user,
-                                            tenant_uuid="foo").list())
-        self.assertFalse(user.neutron.return_value.list_floatingips.called)
-
-        user.services.return_value = {
-            consts.ServiceType.NETWORK: consts.Service.NEUTRON}
         self.assertEqual(fips["floatingips"], list(
             resources.NeutronFloatingIP(user=user, tenant_uuid="foo").list()))
         user.neutron.return_value.list_floatingips.assert_called_once_with(
@@ -478,10 +461,11 @@ class NeutronPortTestCase(test.TestCase):
             "device_id": "dev_id",
         }
 
-        self.assertIsInstance(
+        # automatically created or manually created port. No name field
+        self.assertEqual(
             resources.NeutronPort(resource=raw_res,
                                   user=mock.MagicMock()).name(),
-            resources.base.NoName)
+            "")
 
         raw_res["name"] = "foo"
         self.assertEqual("foo", resources.NeutronPort(
@@ -604,43 +588,49 @@ class NeutronQuotaTestCase(test.TestCase):
 @ddt.ddt
 class GlanceImageTestCase(test.TestCase):
 
-    @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
-    def test__wrapper_admin(self, mock_glance_wrap):
+    @mock.patch("rally.plugins.openstack.services.image.image.Image")
+    def test__client_admin(self, mock_image):
         admin = mock.Mock()
         glance = resources.GlanceImage(admin=admin)
-        wrapper = glance._wrapper()
+        client = glance._client()
 
-        mock_glance_wrap.assert_called_once_with(admin.glance, glance)
-        self.assertEqual(wrapper, mock_glance_wrap.return_value)
+        mock_image.assert_called_once_with(admin)
+        self.assertEqual(client, mock_image.return_value)
 
-    @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
-    def test__wrapper_user(self, mock_glance_wrap):
+    @mock.patch("rally.plugins.openstack.services.image.image.Image")
+    def test__client_user(self, mock_image):
         user = mock.Mock()
         glance = resources.GlanceImage(user=user)
-        wrapper = glance._wrapper()
+        wrapper = glance._client()
 
-        mock_glance_wrap.assert_called_once_with(user.glance, glance)
-        self.assertEqual(wrapper, mock_glance_wrap.return_value)
+        mock_image.assert_called_once_with(user)
+        self.assertEqual(wrapper, mock_image.return_value)
 
-    @mock.patch("rally.plugins.openstack.wrappers.glance.wrap")
-    def test__wrapper_admin_preferred(self, mock_glance_wrap):
+    @mock.patch("rally.plugins.openstack.services.image.image.Image")
+    def test__client_admin_preferred(self, mock_image):
         admin = mock.Mock()
         user = mock.Mock()
         glance = resources.GlanceImage(admin=admin, user=user)
-        wrapper = glance._wrapper()
+        client = glance._client()
 
-        mock_glance_wrap.assert_called_once_with(admin.glance, glance)
-        self.assertEqual(wrapper, mock_glance_wrap.return_value)
+        mock_image.assert_called_once_with(admin)
+        self.assertEqual(client, mock_image.return_value)
 
     def test_list(self):
         glance = resources.GlanceImage()
-        glance._wrapper = mock.Mock()
+        glance._client = mock.Mock()
+        list_images = glance._client.return_value.list_images
+        list_images.side_effect = (
+            ["active-image1", "active-image2"],
+            ["deactivated-image1"])
         glance.tenant_uuid = mock.Mock()
 
-        self.assertEqual(glance.list(),
-                         glance._wrapper.return_value.list_images.return_value)
-        glance._wrapper.return_value.list_images.assert_called_once_with(
-            owner=glance.tenant_uuid)
+        self.assertEqual(
+            glance.list(),
+            ["active-image1", "active-image2", "deactivated-image1"])
+        list_images.assert_has_calls([
+            mock.call(owner=glance.tenant_uuid),
+            mock.call(status="deactivated", owner=glance.tenant_uuid)])
 
     def test_delete(self):
         glance = resources.GlanceImage()
@@ -649,13 +639,30 @@ class GlanceImageTestCase(test.TestCase):
         glance.raw_resource = mock.Mock()
 
         client = glance._client.return_value
-        wrapper = glance._wrapper.return_value
 
         deleted_image = mock.Mock(status="DELETED")
-        wrapper.get_image.side_effect = [glance.raw_resource, deleted_image]
+        client.get_image.side_effect = [glance.raw_resource, deleted_image]
 
         glance.delete()
-        client().images.delete.assert_called_once_with(glance.raw_resource.id)
+        client.delete_image.assert_called_once_with(glance.raw_resource.id)
+        self.assertFalse(client.reactivate_image.called)
+
+    @mock.patch("%s.reactivate_image" % GLANCE_V2_PATH)
+    def test_delete_deactivated_image(self, mock_reactivate_image):
+        glance = resources.GlanceImage()
+        glance._client = mock.Mock()
+        glance._wrapper = mock.Mock()
+        glance.raw_resource = mock.Mock(status="deactivated")
+
+        client = glance._client.return_value
+
+        deleted_image = mock.Mock(status="DELETED")
+        client.get_image.side_effect = [glance.raw_resource, deleted_image]
+
+        glance.delete()
+
+        mock_reactivate_image.assert_called_once_with(glance.raw_resource.id)
+        client.delete_image.assert_called_once_with(glance.raw_resource.id)
 
 
 class CeilometerTestCase(test.TestCase):
@@ -901,57 +908,29 @@ class ManilaSecurityServiceTestCase(test.TestCase):
             "fake_id")
 
 
-class MistralMixinTestCase(test.TestCase):
-
-    def test_delete(self):
-        mistral = resources.MistralMixin()
-        mistral._service = "mistral"
-        mistral.user = mock.MagicMock()
-        mistral._resource = "some_resources"
-        mistral.raw_resource = {"id": "TEST_ID"}
-        mistral.user.mistral().some_resources.delete.return_value = None
-
-        mistral.delete()
-        mistral.user.mistral().some_resources.delete.assert_called_once_with(
-            "TEST_ID")
-
-
 class MistralWorkbookTestCase(test.TestCase):
 
     def test_delete(self):
-        mistral = resources.MistralWorkbooks()
-        mistral._service = "mistral"
-        mistral.user = mock.MagicMock()
-        mistral._resource = "some_resources"
-        mistral.raw_resource = {"name": "TEST_NAME"}
-        mistral.user.mistral().some_resources.delete.return_value = None
+        clients = mock.MagicMock()
+        resource = mock.Mock()
+        resource.name = "TEST_NAME"
+
+        mistral = resources.MistralWorkbooks(
+            user=clients,
+            resource=resource)
 
         mistral.delete()
-        mistral.user.mistral().some_resources.delete.assert_called_once_with(
+
+        clients.mistral().workbooks.delete.assert_called_once_with(
             "TEST_NAME")
 
 
-class FuelEnvironmentTestCase(test.TestCase):
-
-    def test_id(self):
-        fres = resources.FuelEnvironment()
-        fres.raw_resource = {"id": 42, "name": "chavez"}
-        self.assertEqual(42, fres.id())
+class MistralExecutionsTestCase(test.TestCase):
 
     def test_name(self):
-        fuel = resources.FuelEnvironment()
-        fuel.raw_resource = {"id": "test_id", "name": "test_name"}
-        self.assertEqual("test_name", fuel.name())
-
-    @mock.patch("%s.FuelEnvironment._manager" % BASE)
-    def test_is_deleted(self, mock__manager):
-        mock__manager.return_value.get.return_value = None
-        fres = resources.FuelEnvironment()
-        fres.id = mock.Mock()
-        self.assertTrue(fres.is_deleted())
-        mock__manager.return_value.get.return_value = "env"
-        self.assertFalse(fres.is_deleted())
-        mock__manager.return_value.get.assert_called_with(fres.id.return_value)
+        execution = mock.MagicMock(workflow_name="bar")
+        execution.name = "foo"
+        self.assertEqual("bar", resources.MistralExecutions(execution).name())
 
 
 class SenlinMixinTestCase(test.TestCase):
@@ -1058,8 +1037,7 @@ class WatcherActionPlanTestCase(test.TestCase):
 
     def test_name(self):
         watcher = resources.WatcherActionPlan()
-        watcher.raw_resource = mock.MagicMock(uuid="name")
-        self.assertEqual("name", watcher.name())
+        self.assertIsInstance(watcher.name(), resources.base.NoName)
 
     @mock.patch("%s.WatcherActionPlan._manager" % BASE)
     def test_is_deleted(self, mock__manager):
@@ -1078,3 +1056,47 @@ class WatcherActionPlanTestCase(test.TestCase):
 
         self.assertEqual("action_plan", watcher._resource)
         watcher._manager().list.assert_called_once_with(limit=0)
+
+
+class CinderImageVolumeCacheTestCase(test.TestCase):
+
+    class Resource(object):
+        def __init__(self, id=None, name=None):
+            self.id = id
+            self.name = name
+
+    @mock.patch("rally.plugins.openstack.services.image.image.Image")
+    def test_list(self, mock_image):
+        admin = mock.Mock()
+
+        glance = mock_image.return_value
+        cinder = admin.cinder.return_value
+
+        image_1 = self.Resource("foo", name="foo-name")
+        image_2 = self.Resource("bar", name="bar-name")
+        glance.list_images.return_value = [image_1, image_2]
+        volume_1 = self.Resource(name="v1")
+        volume_2 = self.Resource(name="image-foo")
+        volume_3 = self.Resource(name="foo")
+        volume_4 = self.Resource(name="bar")
+        cinder.volumes.list.return_value = [volume_1, volume_2, volume_3,
+                                            volume_4]
+
+        manager = resources.CinderImageVolumeCache(admin=admin)
+
+        self.assertEqual([{"volume": volume_2, "image": image_1}],
+                         manager.list())
+
+        mock_image.assert_called_once_with(admin)
+        glance.list_images.assert_called_once_with()
+        cinder.volumes.list.assert_called_once_with(
+            search_opts={"all_tenants": 1})
+
+    def test_id_and_name(self):
+
+        res = resources.CinderImageVolumeCache(
+            {"volume": self.Resource("volume-id", "volume-name"),
+             "image": self.Resource("image-id", "image-name")})
+
+        self.assertEqual("volume-id", res.id())
+        self.assertEqual("image-name", res.name())
