@@ -13,16 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import ddt
 import mock
 
 from rally.common import utils
 from rally import consts
-from rally import exceptions
 from rally.plugins.openstack.wrappers import network
 from tests.unit import test
 
 from neutronclient.common import exceptions as neutron_exceptions
-from novaclient import exceptions as nova_exceptions
 
 SVC = "rally.plugins.openstack.wrappers.network."
 
@@ -31,149 +30,7 @@ class Owner(utils.RandomNameGeneratorMixin):
     task = {"uuid": "task-uuid"}
 
 
-class NovaNetworkWrapperTestCase(test.TestCase):
-
-    class Net(object):
-        def __init__(self, **kwargs):
-            if "tenant_id" in kwargs:
-                kwargs["project_id"] = kwargs.pop("tenant_id")
-            self.__dict__.update(kwargs)
-
-    def setUp(self):
-        self.owner = Owner()
-        self.owner.generate_random_name = mock.Mock()
-        super(NovaNetworkWrapperTestCase, self).setUp()
-
-    def get_wrapper(self, *skip_cidrs, **kwargs):
-        mock_clients = mock.Mock()
-        mock_clients.nova.return_value.networks.list.return_value = [
-            self.Net(cidr=cidr) for cidr in skip_cidrs]
-        return network.NovaNetworkWrapper(mock_clients, self.owner,
-                                          config=kwargs)
-
-    def test__init__(self):
-        skip_cidrs = ["foo_cidr", "bar_cidr"]
-        service = self.get_wrapper(*skip_cidrs)
-        self.assertEqual(service.skip_cidrs, skip_cidrs)
-        service.client.networks.list.assert_called_once_with()
-
-    @mock.patch("rally.plugins.openstack.wrappers.network.generate_cidr")
-    def test__generate_cidr(self, mock_generate_cidr):
-        skip_cidrs = [5, 7]
-        cidrs = iter(range(7))
-        mock_generate_cidr.side_effect = (
-            lambda start_cidr: start_cidr + next(cidrs)
-        )
-        service = self.get_wrapper(*skip_cidrs, start_cidr=3)
-        self.assertEqual(service._generate_cidr(), 3)
-        self.assertEqual(service._generate_cidr(), 4)
-        self.assertEqual(service._generate_cidr(), 6)  # 5 is skipped
-        self.assertEqual(service._generate_cidr(), 8)  # 7 is skipped
-        self.assertEqual(service._generate_cidr(), 9)
-        self.assertEqual(mock_generate_cidr.mock_calls,
-                         [mock.call(start_cidr=3)] * 7)
-
-    def test_create_network(self):
-        service = self.get_wrapper()
-        service.client.networks.create.side_effect = (
-            lambda **kwargs: self.Net(id="foo_id", **kwargs))
-        service._generate_cidr = mock.Mock(return_value="foo_cidr")
-        net = service.create_network("foo_tenant",
-                                     network_create_args={"fakearg": "fake"},
-                                     bar="spam")
-        self.assertEqual(net,
-                         {"id": "foo_id",
-                          "name": self.owner.generate_random_name.return_value,
-                          "cidr": "foo_cidr",
-                          "status": "ACTIVE",
-                          "external": False,
-                          "tenant_id": "foo_tenant"})
-        service._generate_cidr.assert_called_once_with()
-        service.client.networks.create.assert_called_once_with(
-            project_id="foo_tenant", cidr="foo_cidr",
-            label=self.owner.generate_random_name.return_value,
-            fakearg="fake")
-
-    def test_delete_network(self):
-        service = self.get_wrapper()
-        service.client.networks.delete.return_value = "foo_deleted"
-        self.assertEqual(service.delete_network({"id": "foo_id"}),
-                         "foo_deleted")
-        service.client.networks.disassociate.assert_called_once_with(
-            "foo_id", disassociate_host=False, disassociate_project=True)
-        service.client.networks.delete.assert_called_once_with("foo_id")
-
-    def test_list_networks(self):
-        service = self.get_wrapper()
-        service.client.networks.list.reset_mock()
-        service.client.networks.list.return_value = [
-            self.Net(id="foo_id", project_id="foo_tenant", cidr="foo_cidr",
-                     label="foo_label"),
-            self.Net(id="bar_id", project_id="bar_tenant", cidr="bar_cidr",
-                     label="bar_label")]
-        expected = [
-            {"id": "foo_id", "cidr": "foo_cidr", "name": "foo_label",
-             "status": "ACTIVE", "external": False, "tenant_id": "foo_tenant"},
-            {"id": "bar_id", "cidr": "bar_cidr", "name": "bar_label",
-             "status": "ACTIVE", "external": False, "tenant_id": "bar_tenant"}]
-        self.assertEqual(expected, service.list_networks())
-        service.client.networks.list.assert_called_once_with()
-
-    def test__get_floating_ip(self):
-        wrap = self.get_wrapper()
-        wrap.client.floating_ips.get.return_value = mock.Mock(id="foo_id",
-                                                              ip="foo_ip")
-        fip = wrap._get_floating_ip("fip_id")
-        wrap.client.floating_ips.get.assert_called_once_with("fip_id")
-        self.assertEqual(fip, "foo_id")
-
-        wrap.client.floating_ips.get.side_effect = (
-            nova_exceptions.NotFound(""))
-        self.assertIsNone(wrap._get_floating_ip("fip_id"))
-
-        self.assertRaises(exceptions.GetResourceNotFound,
-                          wrap._get_floating_ip, "fip_id", do_raise=True)
-
-    def test_create_floating_ip(self):
-        wrap = self.get_wrapper()
-        wrap.client.floating_ips.create.return_value = mock.Mock(id="foo_id",
-                                                                 ip="foo_ip")
-        fip = wrap.create_floating_ip(ext_network="bar_net", bar="spam")
-        self.assertEqual(fip, {"ip": "foo_ip", "id": "foo_id"})
-        wrap.client.floating_ips.create.assert_called_once_with("bar_net")
-
-        net = mock.Mock()
-        net.name = "foo_net"
-        wrap.client.floating_ip_pools.list.return_value = [net]
-        fip = wrap.create_floating_ip()
-        self.assertEqual(fip, {"ip": "foo_ip", "id": "foo_id"})
-        wrap.client.floating_ips.create.assert_called_with("foo_net")
-
-    def test_delete_floating_ip(self):
-        wrap = self.get_wrapper()
-        fip_found = iter(range(3))
-
-        def get_fip(*args, **kwargs):
-            for i in fip_found:
-                return "fip_id"
-            raise exceptions.GetResourceNotFound(resource="")
-        wrap._get_floating_ip = mock.Mock(side_effect=get_fip)
-
-        wrap.delete_floating_ip("fip_id")
-        wrap.client.floating_ips.delete.assert_called_once_with("fip_id")
-        self.assertFalse(wrap._get_floating_ip.called)
-
-        wrap.delete_floating_ip("fip_id", wait=True)
-        self.assertEqual(
-            [mock.call("fip_id", do_raise=True)] * 4,
-            wrap._get_floating_ip.mock_calls)
-
-    def test_supports_extension(self):
-        wrap = self.get_wrapper()
-        self.assertFalse(wrap.supports_extension("extension")[0])
-        self.assertTrue(wrap.supports_extension("security-group")[0])
-
-
+@ddt.ddt
 class NeutronWrapperTestCase(test.TestCase):
     def setUp(self):
         self.owner = Owner()
@@ -184,7 +41,7 @@ class NeutronWrapperTestCase(test.TestCase):
         return network.NeutronWrapper(mock.Mock(), self.owner, config=kwargs)
 
     def test_SUBNET_IP_VERSION(self):
-        self.assertEqual(network.NeutronWrapper.SUBNET_IP_VERSION, 4)
+        self.assertEqual(4, network.NeutronWrapper.SUBNET_IP_VERSION)
 
     @mock.patch("rally.plugins.openstack.wrappers.network.generate_cidr")
     def test__generate_cidr(self, mock_generate_cidr):
@@ -193,18 +50,18 @@ class NeutronWrapperTestCase(test.TestCase):
             lambda start_cidr: start_cidr + next(cidrs)
         )
         service = self.get_wrapper(start_cidr=3)
-        self.assertEqual(service._generate_cidr(), 3)
-        self.assertEqual(service._generate_cidr(), 4)
-        self.assertEqual(service._generate_cidr(), 5)
-        self.assertEqual(service._generate_cidr(), 6)
-        self.assertEqual(service._generate_cidr(), 7)
-        self.assertEqual(mock_generate_cidr.mock_calls,
-                         [mock.call(start_cidr=3)] * 5)
+        self.assertEqual(3, service._generate_cidr())
+        self.assertEqual(4, service._generate_cidr())
+        self.assertEqual(5, service._generate_cidr())
+        self.assertEqual(6, service._generate_cidr())
+        self.assertEqual(7, service._generate_cidr())
+        self.assertEqual([mock.call(start_cidr=3)] * 5,
+                         mock_generate_cidr.mock_calls)
 
     def test_external_networks(self):
         wrap = self.get_wrapper()
         wrap.client.list_networks.return_value = {"networks": "foo_networks"}
-        self.assertEqual(wrap.external_networks, "foo_networks")
+        self.assertEqual("foo_networks", wrap.external_networks)
         wrap.client.list_networks.assert_called_once_with(
             **{"router:external": True})
 
@@ -225,7 +82,7 @@ class NeutronWrapperTestCase(test.TestCase):
                         "subnets": "foo_subnets"}
         wrap.client.show_network.return_value = {"network": neutron_net}
         net = wrap.get_network(net_id="foo_id")
-        self.assertEqual(net, expected_net)
+        self.assertEqual(expected_net, net)
         wrap.client.show_network.assert_called_once_with("foo_id")
 
         wrap.client.show_network.side_effect = (
@@ -235,7 +92,7 @@ class NeutronWrapperTestCase(test.TestCase):
 
         wrap.client.list_networks.return_value = {"networks": [neutron_net]}
         net = wrap.get_network(name="foo_name")
-        self.assertEqual(net, expected_net)
+        self.assertEqual(expected_net, net)
         wrap.client.list_networks.assert_called_once_with(name="foo_name")
 
         wrap.client.list_networks.return_value = {"networks": []}
@@ -259,7 +116,7 @@ class NeutronWrapperTestCase(test.TestCase):
                      "tenant_id": tenant,
                      "protocol": "HTTP",
                      "name": self.owner.generate_random_name.return_value}})
-        self.assertEqual(resultant_pool, expected_pool)
+        self.assertEqual(expected_pool, resultant_pool)
 
     def test_create_network(self):
         service = self.get_wrapper()
@@ -271,14 +128,13 @@ class NeutronWrapperTestCase(test.TestCase):
         service.client.create_network.assert_called_once_with({
             "network": {"tenant_id": "foo_tenant",
                         "name": self.owner.generate_random_name.return_value}})
-        self.assertEqual(net,
-                         {"id": "foo_id",
+        self.assertEqual({"id": "foo_id",
                           "name": self.owner.generate_random_name.return_value,
                           "status": "foo_status",
                           "external": False,
                           "tenant_id": "foo_tenant",
                           "router_id": None,
-                          "subnets": []})
+                          "subnets": []}, net)
 
     def test_create_network_with_subnets(self):
         subnets_num = 4
@@ -300,15 +156,14 @@ class NeutronWrapperTestCase(test.TestCase):
         service.client.create_network.assert_called_once_with({
             "network": {"tenant_id": "foo_tenant",
                         "name": self.owner.generate_random_name.return_value}})
-        self.assertEqual(net,
-                         {"id": "foo_id",
+        self.assertEqual({"id": "foo_id",
                           "name": self.owner.generate_random_name.return_value,
                           "status": "foo_status",
                           "external": False,
                           "router_id": None,
                           "tenant_id": "foo_tenant",
                           "subnets": ["subnet-%d" % i
-                                      for i in range(subnets_num)]})
+                                      for i in range(subnets_num)]}, net)
         self.assertEqual(
             service.client.create_subnet.mock_calls,
             [mock.call({"subnet":
@@ -329,14 +184,13 @@ class NeutronWrapperTestCase(test.TestCase):
                         "name": self.owner.generate_random_name.return_value,
                         "status": "foo_status"}}
         net = service.create_network("foo_tenant", add_router=True)
-        self.assertEqual(net,
-                         {"id": "foo_id",
+        self.assertEqual({"id": "foo_id",
                           "name": self.owner.generate_random_name.return_value,
                           "status": "foo_status",
                           "external": False,
                           "tenant_id": "foo_tenant",
                           "router_id": "foo_router",
-                          "subnets": []})
+                          "subnets": []}, net)
         service.create_router.assert_called_once_with(external=True,
                                                       tenant_id="foo_tenant")
 
@@ -354,14 +208,13 @@ class NeutronWrapperTestCase(test.TestCase):
         net = service.create_network("foo_tenant", add_router=True,
                                      subnets_num=subnets_num,
                                      dns_nameservers=["foo_nameservers"])
-        self.assertEqual(net,
-                         {"id": "foo_id",
+        self.assertEqual({"id": "foo_id",
                           "name": self.owner.generate_random_name.return_value,
                           "status": "foo_status",
                           "external": False,
                           "tenant_id": "foo_tenant",
                           "router_id": "foo_router",
-                          "subnets": ["foo_subnet"] * subnets_num})
+                          "subnets": ["foo_subnet"] * subnets_num}, net)
         service.create_router.assert_called_once_with(external=True,
                                                       tenant_id="foo_tenant")
         self.assertEqual(
@@ -386,12 +239,12 @@ class NeutronWrapperTestCase(test.TestCase):
         service.client.delete_network.return_value = "foo_deleted"
         result = service.delete_network({"id": "foo_id", "router_id": None,
                                          "subnets": []})
-        self.assertEqual(result, "foo_deleted")
-        self.assertEqual(service.client.remove_gateway_router.mock_calls, [])
+        self.assertEqual("foo_deleted", result)
+        self.assertEqual([], service.client.remove_gateway_router.mock_calls)
         self.assertEqual(
-            service.client.remove_interface_router.mock_calls, [])
-        self.assertEqual(service.client.delete_router.mock_calls, [])
-        self.assertEqual(service.client.delete_subnet.mock_calls, [])
+            [], service.client.remove_interface_router.mock_calls)
+        self.assertEqual([], service.client.delete_router.mock_calls)
+        self.assertEqual([], service.client.delete_subnet.mock_calls)
         service.client.delete_network.assert_called_once_with("foo_id")
 
     def test_delete_v1_pool(self):
@@ -420,7 +273,7 @@ class NeutronWrapperTestCase(test.TestCase):
             {"id": "foo_id", "router_id": "foo_router", "subnets": subnets,
              "lb_pools": []})
 
-        self.assertEqual(result, "foo_deleted")
+        self.assertEqual("foo_deleted", result)
         self.assertEqual(
             service.client.remove_network_from_dhcp_agent.mock_calls,
             [mock.call(agent_id, "foo_id") for agent_id in agents])
@@ -436,10 +289,67 @@ class NeutronWrapperTestCase(test.TestCase):
         mock_neutron_wrapper_supports_extension.assert_called_once_with(
             "dhcp_agent_scheduler")
 
+    @ddt.data({"exception_type": neutron_exceptions.NotFound,
+               "should_raise": False},
+              {"exception_type": neutron_exceptions.BadRequest,
+               "should_raise": False},
+              {"exception_type": KeyError,
+               "should_raise": True})
+    @ddt.unpack
+    @mock.patch("rally.plugins.openstack.wrappers.network.NeutronWrapper"
+                ".supports_extension", return_value=(True, ""))
+    def test_delete_network_with_router_throw_exception(
+            self, mock_neutron_wrapper_supports_extension, exception_type,
+            should_raise):
+        # Ensure cleanup context still move forward even
+        # remove_interface_router throw NotFound/BadRequest exception
+
+        service = self.get_wrapper()
+        service.client.remove_interface_router.side_effect = exception_type
+        agents = ["foo_agent", "bar_agent"]
+        subnets = ["foo_subnet", "bar_subnet"]
+        ports = [{"id": "foo_port", "device_owner": "network:router_interface",
+                  "device_id": "rounttter"},
+                 {"id": "bar_port", "device_owner": "network:dhcp"}]
+        service.client.list_dhcp_agent_hosting_networks.return_value = (
+            {"agents": [{"id": agent_id} for agent_id in agents]})
+        service.client.list_ports.return_value = ({"ports": ports})
+        service.client.delete_network.return_value = "foo_deleted"
+
+        if should_raise:
+            self.assertRaises(exception_type, service.delete_network,
+                              {"id": "foo_id", "router_id": "foo_router",
+                               "subnets": subnets, "lb_pools": []})
+
+            self.assertNotEqual(service.client.delete_subnet.mock_calls,
+                                [mock.call(subnet_id) for subnet_id in
+                                 subnets])
+            self.assertFalse(service.client.delete_network.called)
+        else:
+            result = service.delete_network(
+                {"id": "foo_id", "router_id": "foo_router", "subnets": subnets,
+                 "lb_pools": []})
+
+            self.assertEqual("foo_deleted", result)
+            service.client.delete_port.assert_called_once_with(ports[1]["id"])
+            service.client.remove_interface_router.assert_called_once_with(
+                ports[0]["device_id"], {"port_id": ports[0]["id"]})
+            self.assertEqual(service.client.delete_subnet.mock_calls,
+                             [mock.call(subnet_id) for subnet_id in subnets])
+            service.client.delete_network.assert_called_once_with("foo_id")
+
+        self.assertEqual(
+            service.client.remove_network_from_dhcp_agent.mock_calls,
+            [mock.call(agent_id, "foo_id") for agent_id in agents])
+        self.assertEqual(service.client.remove_gateway_router.mock_calls,
+                         [mock.call("foo_router")])
+        mock_neutron_wrapper_supports_extension.assert_called_once_with(
+            "dhcp_agent_scheduler")
+
     def test_list_networks(self):
         service = self.get_wrapper()
         service.client.list_networks.return_value = {"networks": "foo_nets"}
-        self.assertEqual(service.list_networks(), "foo_nets")
+        self.assertEqual("foo_nets", service.list_networks())
         service.client.list_networks.assert_called_once_with()
 
     @mock.patch(SVC + "NeutronWrapper.external_networks")
@@ -461,7 +371,7 @@ class NeutronWrapperTestCase(test.TestCase):
         )
         fip = wrap.create_floating_ip(tenant_id="foo_tenant",
                                       port_id="port_id")
-        self.assertEqual(fip, {"id": "fip_id", "ip": "fip_ip"})
+        self.assertEqual({"id": "fip_id", "ip": "fip_ip"}, fip)
 
         wrap.get_network = mock.Mock(
             return_value={"id": "foo_net", "external": True})
@@ -480,13 +390,38 @@ class NeutronWrapperTestCase(test.TestCase):
         wrap = self.get_wrapper()
         wrap.delete_floating_ip("fip_id")
         wrap.delete_floating_ip("fip_id", ignored_kwarg="bar")
-        self.assertEqual(wrap.client.delete_floatingip.mock_calls,
-                         [mock.call("fip_id")] * 2)
+        self.assertEqual([mock.call("fip_id")] * 2,
+                         wrap.client.delete_floatingip.mock_calls)
 
     @mock.patch(SVC + "NeutronWrapper.external_networks")
     def test_create_router(self, mock_neutron_wrapper_external_networks):
         wrap = self.get_wrapper()
         wrap.client.create_router.return_value = {"router": "foo_router"}
+        wrap.client.list_extensions.return_value = {
+            "extensions": [{"alias": "ext-gw-mode"}]}
+        mock_neutron_wrapper_external_networks.__get__ = (
+            lambda *args: [{"id": "ext_id"}]
+        )
+
+        router = wrap.create_router()
+        wrap.client.create_router.assert_called_once_with(
+            {"router": {"name": self.owner.generate_random_name.return_value}})
+        self.assertEqual("foo_router", router)
+
+        router = wrap.create_router(external=True, foo="bar")
+        wrap.client.create_router.assert_called_with(
+            {"router": {"name": self.owner.generate_random_name.return_value,
+                        "external_gateway_info": {
+                            "network_id": "ext_id",
+                            "enable_snat": True},
+                        "foo": "bar"}})
+
+    @mock.patch(SVC + "NeutronWrapper.external_networks")
+    def test_create_router_without_ext_gw_mode_extension(
+            self, mock_neutron_wrapper_external_networks):
+        wrap = self.get_wrapper()
+        wrap.client.create_router.return_value = {"router": "foo_router"}
+        wrap.client.list_extensions.return_value = {"extensions": []}
         mock_neutron_wrapper_external_networks.__get__ = (
             lambda *args: [{"id": "ext_id"}]
         )
@@ -499,9 +434,7 @@ class NeutronWrapperTestCase(test.TestCase):
         router = wrap.create_router(external=True, foo="bar")
         wrap.client.create_router.assert_called_with(
             {"router": {"name": self.owner.generate_random_name.return_value,
-                        "external_gateway_info": {
-                            "network_id": "ext_id",
-                            "enable_snat": True},
+                        "external_gateway_info": {"network_id": "ext_id"},
                         "foo": "bar"}})
 
     def test_create_port(self):
@@ -512,7 +445,7 @@ class NeutronWrapperTestCase(test.TestCase):
         wrap.client.create_port.assert_called_once_with(
             {"port": {"network_id": "foo_net",
                       "name": self.owner.generate_random_name.return_value}})
-        self.assertEqual(port, "foo_port")
+        self.assertEqual("foo_port", port)
 
         port = wrap.create_port("foo_net", foo="bar")
         wrap.client.create_port.assert_called_with(
@@ -539,16 +472,16 @@ class FunctionsTestCase(test.TestCase):
     def test_generate_cidr(self):
         with mock.patch("rally.plugins.openstack.wrappers.network.cidr_incr",
                         iter(range(1, 4))):
-            self.assertEqual(network.generate_cidr(), "10.2.1.0/24")
-            self.assertEqual(network.generate_cidr(), "10.2.2.0/24")
-            self.assertEqual(network.generate_cidr(), "10.2.3.0/24")
+            self.assertEqual("10.2.1.0/24", network.generate_cidr())
+            self.assertEqual("10.2.2.0/24", network.generate_cidr())
+            self.assertEqual("10.2.3.0/24", network.generate_cidr())
 
         with mock.patch("rally.plugins.openstack.wrappers.network.cidr_incr",
                         iter(range(1, 4))):
             start_cidr = "1.1.0.0/26"
-            self.assertEqual(network.generate_cidr(start_cidr), "1.1.0.64/26")
-            self.assertEqual(network.generate_cidr(start_cidr), "1.1.0.128/26")
-            self.assertEqual(network.generate_cidr(start_cidr), "1.1.0.192/26")
+            self.assertEqual("1.1.0.64/26", network.generate_cidr(start_cidr))
+            self.assertEqual("1.1.0.128/26", network.generate_cidr(start_cidr))
+            self.assertEqual("1.1.0.192/26", network.generate_cidr(start_cidr))
 
     def test_wrap(self):
         mock_clients = mock.Mock()
@@ -561,9 +494,3 @@ class FunctionsTestCase(test.TestCase):
         self.assertIsInstance(wrapper, network.NeutronWrapper)
         self.assertEqual(wrapper.owner, owner)
         self.assertEqual(wrapper.config, config)
-
-        mock_clients.services.return_value = {"foo": "bar"}
-        wrapper = network.wrap(mock_clients, owner, config)
-        self.assertIsInstance(wrapper, network.NovaNetworkWrapper)
-        self.assertEqual(wrapper.owner, owner)
-        self.assertEqual(wrapper.config, config)
