@@ -13,10 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import datetime as dt
 
+import ddt
 from jsonschema import exceptions as schema_exceptions
 import mock
+import six
 
 from rally import exceptions
 from rally.task import utils
@@ -24,7 +27,7 @@ from tests.unit import fakes
 from tests.unit import test
 
 
-class BenchmarkUtilsTestCase(test.TestCase):
+class TaskUtilsTestCase(test.TestCase):
 
     def test_wait_for_delete(self):
         def update_resource(self):
@@ -84,14 +87,14 @@ class BenchmarkUtilsTestCase(test.TestCase):
         manager = fakes.FakeManager()
         resource = fakes.FakeResource(manager=manager)
         manager._cache(resource)
-        self.assertEqual(get_from_manager(resource), resource)
+        self.assertEqual(resource, get_from_manager(resource))
 
     def test_get_from_manager_with_uuid_field(self):
         get_from_manager = utils.get_from_manager()
         manager = fakes.FakeManager()
         resource = fakes.FakeResource(manager=manager)
         manager._cache(resource)
-        self.assertEqual(get_from_manager(resource, id_attr="uuid"), resource)
+        self.assertEqual(resource, get_from_manager(resource, id_attr="uuid"))
 
     def test_get_from_manager_in_error_state(self):
         get_from_manager = utils.get_from_manager()
@@ -235,10 +238,10 @@ class WaitForTestCase(test.TestCase):
             update_resource=self.fake_updater, timeout=self.load_secs,
             check_interval=self.load_secs / 3)
 
-        self.assertEqual(exc.kwargs["resource_name"], "fake_name")
-        self.assertEqual(exc.kwargs["resource_id"], "fake_id")
-        self.assertEqual(exc.kwargs["desired_status"], "fake_new_status")
-        self.assertEqual(exc.kwargs["resource_status"], "FAKE_STALE_STATUS")
+        self.assertEqual("fake_name", exc.kwargs["resource_name"])
+        self.assertEqual("fake_id", exc.kwargs["resource_id"])
+        self.assertEqual("fake_new_status", exc.kwargs["desired_status"])
+        self.assertEqual("FAKE_STALE_STATUS", exc.kwargs["resource_status"])
 
         self.assertIn("FakeResource", str(exc))
         self.assertIn("fake_new_status", str(exc))
@@ -514,9 +517,10 @@ class WaitForStatusTestCase(test.TestCase):
                                     update_resource=upd)
         self.assertEqual(res, ret)
 
+    @mock.patch("rally.task.utils.LOG.debug")
     @mock.patch("rally.task.utils.time.sleep")
     @mock.patch("rally.task.utils.time.time", side_effect=[1, 2, 3, 4])
-    def test_wait_timeout(self, mock_time, mock_sleep):
+    def test_wait_timeout(self, mock_time, mock_sleep, mock_log_debug):
         res = {"status": "not_ready"}
         upd = mock.MagicMock(side_effect=[{"status": "not_ready"},
                                           {"status": "fail"}])
@@ -524,3 +528,80 @@ class WaitForStatusTestCase(test.TestCase):
                           utils.wait_for_status,
                           resource=res, ready_statuses=["ready"],
                           update_resource=upd, timeout=2, id_attr="uuid")
+
+
+@ddt.ddt
+class WrapperForAtomicActionsTestCase(test.TestCase):
+
+    def test_dict_atomic(self):
+        atomic_actions = collections.OrderedDict(
+            [("action_1", 1), ("action_2", 2)])
+        atomic_wrapper = utils.WrapperForAtomicActions(atomic_actions, 1)
+        self.assertEqual(1, atomic_wrapper["action_1"])
+        self.assertEqual(2, atomic_wrapper["action_2"])
+        self.assertEqual(atomic_actions.items(),
+                         atomic_wrapper.items())
+        self.assertEqual(1, atomic_wrapper.get("action_1"))
+        self.assertIsNone(atomic_wrapper.get("action_3"))
+        self.assertEqual(2, len(atomic_wrapper))
+        self.assertEqual([{"name": "action_1", "started_at": 1,
+                           "finished_at": 2, "children": []},
+                          {"name": "action_2", "started_at": 2,
+                           "finished_at": 4, "children": []}
+                          ], atomic_wrapper)
+
+    def test_list_atomic(self):
+        atomic_actions = [{"name": "action_1", "started_at": 1,
+                           "finished_at": 2, "children": []},
+                          {"name": "action_2", "started_at": 2,
+                           "finished_at": 4, "children": []}]
+
+        atomic_wrapper = utils.WrapperForAtomicActions(atomic_actions)
+        self.assertEqual(1, atomic_wrapper["action_1"])
+        self.assertEqual(2, atomic_wrapper["action_2"])
+        self.assertEqual(
+            collections.OrderedDict(
+                [("action_1", 1), ("action_2", 2)]).items(),
+            atomic_wrapper.items())
+        self.assertEqual(atomic_actions[0], atomic_wrapper[0])
+        self.assertEqual(atomic_actions[1], atomic_wrapper[1])
+        self.assertEqual(1, atomic_wrapper.get("action_1"))
+        self.assertIsNone(atomic_wrapper.get("action_3"))
+        self.assertEqual(2, len(atomic_wrapper))
+        self.assertEqual(atomic_actions[0], six.next(iter(atomic_wrapper)))
+
+    def test__convert_new_atomic_actions(self):
+        atomic_actions = collections.OrderedDict(
+            [("action_1", 1), ("action_2", 2)])
+        atomic_wrapper = utils.WrapperForAtomicActions(atomic_actions)
+        self.assertEqual(
+            [{"name": "action_1", "started_at": 0,
+              "finished_at": 1, "children": []},
+             {"name": "action_2", "started_at": 1,
+              "finished_at": 3, "children": []}],
+            atomic_wrapper._convert_old_atomic_actions(atomic_actions))
+
+    @ddt.data(
+        {"atomic_actions": [{"name": "some", "started_at": 1.0,
+                             "finished_at": 2.0, "children": []}],
+         "expected": {"some": 1.0}},
+        {"atomic_actions": [{"name": "some", "started_at": 1.0,
+                             "finished_at": 2.0, "children": []},
+                            {"name": "some", "started_at": 2.0,
+                             "finished_at": 3.0, "children": []}],
+         "expected": {"some": 1.0, "some (2)": 1.0}},
+        {"atomic_actions": [{"name": "some", "started_at": 1.0,
+                             "finished_at": 2.0, "children": []},
+                            {"name": "some", "started_at": 2.0,
+                             "finished_at": 3.0, "children": []},
+                            {"name": "some", "started_at": 3.0,
+                             "finished_at": 4.0, "children": []}
+                            ],
+         "expected": {"some": 1.0, "some (2)": 1.0, "some (3)": 1.0}}
+    )
+    @ddt.unpack
+    def test_convert_new_atomic_actions(self, atomic_actions, expected):
+        atomic_wrapper = utils.WrapperForAtomicActions(atomic_actions)
+        self.assertEqual(expected,
+                         atomic_wrapper._convert_new_atomic_actions(
+                             atomic_actions))
